import logging
from shutil import which

from zygoat.components import Component
from zygoat.constants import Phases, Projects
from zygoat.utils.files import use_dir
from zygoat.utils.shell import run

log = logging.getLogger()


class Reformat(Component):
    def create(self):
        with use_dir(Projects.BACKEND):
            black = which("black")
            if black is not None:
                run([black, "."])

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    @property
    def installed(self):
        return False  # Always run this, no matter what


reformat = Reformat()



import logging
import os

from . import Component
from . import resources

from importlib import resources as il_resources

log = logging.getLogger()
file_name = "docker-compose.yml"


class DockerCompose(Component):
    def create(self):
        log.info("Creating an empty docker-compose.yml file")

        # All phases and checks run from the repository root
        with open(file_name, "w") as f:
            f.write(il_resources.read_text(resources, file_name))

        log.info(f"Created {file_name} successfully")

    def delete(self):
        log.warning(f"Deleting {file_name}")
        os.remove(file_name)

    @property
    def installed(self):
        return os.path.exists(file_name)


docker_compose = DockerCompose()



from zygoat.components import FileComponent
from zygoat.components.backend import resources


class BanditRC(FileComponent):
    base_path = "backend/tests"
    filename = ".banditrc"
    resource_pkg = resources


banditrc = BanditRC()



from functools import wraps
import logging

from zygoat.constants import Phases
from zygoat.components import Component
from zygoat.constants import Projects
from zygoat.utils.files import use_dir
from zygoat.utils.shell import run

log = logging.getLogger()


class YarnComponent(Component):
    """
    Use this when you want to install a package using yarn.

    :param package: Name of the package to install
    """

    package = None

    def check_setup(f):
        @wraps(f)
        def wrapper(self, *args, **kwargs):
            if not self.package:
                raise NotImplementedError("You must specify cls.package!")
            return f(self, *args, **kwargs)

        return wrapper

    @property
    @check_setup
    def installed(self):
        completed_process = run(["yarn", "info", self.package])
        return completed_process.returncode == 0

    @check_setup
    def create(self):
        with use_dir(Projects.FRONTEND):
            log.info("Installing {self.package} via yarn")
            run(["yarn", "add", self.package])

    @check_setup
    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)
        with use_dir(Projects.FRONTEND):
            log.info("Upgrading {self.package} via yarn")
            run(["yarn", "upgrade", self.package])

    @check_setup
    def delete(self):
        self.call_phase(Phases.CREATE, force_create=True)
        with use_dir(Projects.FRONTEND):
            log.info("Removing {self.package} via yarn")
            run(["yarn", "remove", self.package])



from .base import Component
from .file_component import FileComponent
from .settings_component import SettingsComponent
from . import resources

from .editorconfig import editorconfig
from .codebuild import codebuild
from .script import cistart
from .precommit import precommitconfig
from .docker_compose import docker_compose
from .backend import backend
from .frontend import frontend
from .proxy import reverse_proxy

components = [
    editorconfig,
    precommitconfig,
    docker_compose,
    backend,
    frontend,
    reverse_proxy,
    codebuild,
    cistart,
]

__all__ = [
    Component,
    FileComponent,
    SettingsComponent,
    components,
    resources,
]



import os

from zygoat.constants import Projects
from zygoat.components import FileComponent

from . import resources


class Gitignore(FileComponent):
    filename = ".gitignore"
    resource_pkg = resources
    base_path = os.path.join(Projects.FRONTEND, "cypress")


gitignore = Gitignore()
